package com.cultural.heritage.controller.clothes;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clotheCategory.ClothesCategoryAddRequest;
import com.cultural.heritage.model.dto.clotheCategory.ClothesCategoryUpdateRequest;
import com.cultural.heritage.model.entity.Clothes;
import com.cultural.heritage.model.entity.ClothesCategory;
import com.cultural.heritage.model.vo.clothes.ClothesCategoryVO;
import com.cultural.heritage.model.vo.clothes.ClothesVO;
import com.cultural.heritage.service.clothes.ClothesCategoryService;
import com.cultural.heritage.service.clothes.ClothesService;
import com.cultural.heritage.service.common.CommonService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/clothesCategory")
@Slf4j
@Tag(name = "服装类别管理模块")
public class ClothesCategoryController {

    @Resource
    private ClothesCategoryService clothesCategoryService;


    @Resource
    private CommonService commonService;


    @Resource
    private ClothesService clothesService;



    /**
     * Web端管理员添加服装类别
     * @param clothesCategoryAddRequest 服装类别添加请求体
     * @return 服装类别id
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加服装类别", description = "参数：服装类别添加请求体，权限：管理员(admin, boss)，方法名：addClothesCategory")
    public BaseResponse<Long> addClothesCategory(@RequestBody ClothesCategoryAddRequest clothesCategoryAddRequest) {
        if (clothesCategoryAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ClothesCategory clothesCategory = commonService.copyProperties(clothesCategoryAddRequest, ClothesCategory.class);
        // 校验
        clothesCategoryService.validClothesCategory(clothesCategory, false);
        boolean save = clothesCategoryService.save(clothesCategory);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "服装类别添加失败");
        return ResultUtils.success(clothesCategory.getId());
    }



    /**
     * Web端管理员更新服装类别
     * @param clothesCategoryUpdateRequest 服装类别更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员更新服装类别", description = "参数：服装类别更新请求体，权限：管理员(admin, boss)，方法名：updateClothesCategory")
    public BaseResponse<Boolean> updateClothesCategory(@RequestBody ClothesCategoryUpdateRequest clothesCategoryUpdateRequest) {
        if (clothesCategoryUpdateRequest == null || clothesCategoryUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ClothesCategory clothesCategory = commonService.copyProperties(clothesCategoryUpdateRequest, ClothesCategory.class);
        // 校验
        clothesCategoryService.validClothesCategory(clothesCategory, true);

        boolean result = clothesCategoryService.updateById(clothesCategory);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装类别名称更新失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员根据id删除服装类别
     * @param commonRequest 类别删除请求体
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员根据id删除服装类别", description = "参数：类别删除请求体，权限：管理员(admin, boss)，方法名：deleteClothesCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteClothesCategory(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        // 获取删除条件
        QueryWrapper<Clothes> queryWrapper = commonService.buildQueryWrapperByField("categoryId", id, clothesService);

        // 删除该类别下的所有服装
        clothesService.remove(queryWrapper);

        // 删除服装类别
        boolean result = clothesCategoryService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装类别删除失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员查询服装类别
     * @return 服装类别列表
     */
    @PostMapping("/list/web")
    @Operation(summary = "Web端管理员查询服装类别", description = "参数：无，权限：web端管理员，方法名：listClothesCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<ClothesCategoryVO>> listClothesCategory() {
        List<ClothesCategory> clothesCategoryList = clothesCategoryService.list();
        // 获取服装类别VO列表
        List<ClothesCategoryVO> clothesCategoryVOS = commonService.convertList(clothesCategoryList, ClothesCategoryVO.class);
        return ResultUtils.success(clothesCategoryVOS);
    }




    /**
     * 小程序端用户查询服装类别
     * @return 服装类别列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询服装类别", description = "参数：无，权限：所有人，方法名：listClothesCategoryByUsers")
    public BaseResponse<List<ClothesCategoryVO>> listClothesCategoryByUsers() {
        // 获取已上架的服装
        List<Clothes> clothesList = commonService.findByFieldEqTargetField("isShelves", 1, clothesService);
        // 获取所有存在上架商品的商品类别
        List<ClothesCategory> clothesCategoryList = commonService.findByFieldInTargetField(clothesList, clothesCategoryService, Clothes::getCategoryId, "id");

        List<ClothesCategoryVO> clothesCategoryVOS = commonService.convertList(clothesCategoryList, ClothesCategoryVO.class);
        return ResultUtils.success(clothesCategoryVOS);
    }





    /**
     * 小程序端用户根据类别id查询服装租赁产品
     * @param commonRequest 类别id
     * @return 服装租赁产品列表
     */
    @PostMapping("/list/id")
    @Operation(summary = "小程序端用户根据类别id查询服装租赁产品", description = "参数：类别id，权限：所有人，方法名：listClothesByCategoryId")
    public BaseResponse<List<ClothesVO>> listClothesByCategoryId(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        Map<String, Object> fieldConditions = new HashMap<>();
        fieldConditions.put("categoryId", id);
        fieldConditions.put("isShelves", 1);

        List<Clothes> clothesList = commonService.findByFieldEqTargetFields(fieldConditions, clothesService);
        // 获取服装租赁产品VO列表
        List<ClothesVO> clothesVOS = commonService.convertList(clothesList, ClothesVO.class);
        Collections.reverse(clothesVOS);
        return ResultUtils.success(clothesVOS);
    }


}
