package com.cultural.heritage.controller.file;

import cn.hutool.core.io.FileUtil;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.file.UploadFileRequest;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.enums.FileUploadBizEnum;
import com.cultural.heritage.service.file.IHweiYunOBSService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.RegexUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.Arrays;

@RestController
@RequestMapping("/file")
@Slf4j
@Tag(name = "文件上传接口")
public class FileController {


	@Resource
	private UserService userService;


	@Resource
	private IHweiYunOBSService iHweiYunOBSService;




	/**
	 * Web端文件上传（obs桶）
	 * @param multipartFile	文件对象
	 * @param uploadFileRequest	业务类型
	 * @return	PutObjectResult
	 */
	@PostMapping("/uploadFile")
	@Operation(summary = "Web端文件上传（obs桶）", description = "参数：文件对象（multipartFile), 业务类型(biz)，权限：管理员，方法名：uploadFile")
	public BaseResponse<String> uploadFile(@RequestPart("file") MultipartFile multipartFile, UploadFileRequest uploadFileRequest, HttpServletRequest request) {
		// 校验用户是否登录
		User loginUser = userService.getLoginUser(request);
		if (loginUser == null) {
			throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR, "未登录");
		}
		// 获取文件名
		String filePath = getFilePath(multipartFile, uploadFileRequest);
		// 格式化图片特殊字符
		filePath = RegexUtils.encodeUrl(filePath);
		// 上传文件至华为云服务器
		iHweiYunOBSService.fileUpload(multipartFile, filePath);

		return ResultUtils.success(filePath, "上传成功");
	}



	/**
	 * 文件下载（obs桶)
	 * @param objectKey 具体的文件名（含存储路径）
	 */
	@GetMapping("downloadFile")
	@Operation(summary = "文件下载（obs桶)", description = "参数：文件对象（具体的文件名（含存储路径）)，权限：所有人，方法名：downloadFile")
	public void downloadFile(@RequestParam String objectKey, HttpServletResponse response) {
		iHweiYunOBSService.downloadFile(objectKey, response);
	}




	// 优化：设置一个合理的缓冲区大小
	private static final int BUFFER_SIZE = 8192; // 8 KB


	/**
	 * 文件下载（服务器）
	 * @param filename	文件名
	 * @param response	响应体
	 * @throws IOException
	 */
	@GetMapping("/download/{filename}")
	@Operation(summary = "文件下载（服务器）", description = "参数：文件名，权限：所有人，方法名：downloadFileFromServe")
	public void downloadFileFromServe(@PathVariable String filename, HttpServletResponse response) throws IOException {
		// 文件所在的路径
		String filePath = "/www/Fonts/" + filename;
		File file = new File(filePath);

		// 检查文件是否存在
		if (!file.exists()) {
			response.setStatus(HttpServletResponse.SC_NOT_FOUND);
			return;
		}
		// 设置响应头，告诉浏览器这是一个下载文件
		response.setContentType("application/octet-stream");  // 通用的文件流类型
		response.setHeader("Content-Disposition", "attachment; filename=" + file.getName());
		response.setContentLengthLong(file.length());  // 使用 setContentLengthLong 适应大文件

		// 设置缓存相关的 HTTP 头
		long lastModified = file.lastModified();
		response.setDateHeader("Last-Modified", lastModified); // 文件最后修改时间
		response.setHeader("Cache-Control", "public, max-age=86400");  // 缓存一天（24小时）
		response.setHeader("ETag", String.valueOf(lastModified)); // 使用文件最后修改时间作为 ETag

		// 检查浏览器缓存是否有效
		String ifNoneMatch = response.getHeader("If-None-Match");
		if (ifNoneMatch != null && ifNoneMatch.equals(String.valueOf(lastModified))) {
			// 如果 ETag 匹配，表示文件没有变化，返回 304 Not Modified
			response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
			return;
		}
		// 使用 BufferedInputStream 和 BufferedOutputStream 提高性能
		try (BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file), BUFFER_SIZE);
			 BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream(), BUFFER_SIZE)) {

			byte[] buffer = new byte[BUFFER_SIZE];
			int bytesRead;

			// 从文件中读取数据并写入响应输出流
			while ((bytesRead = bis.read(buffer)) != -1) {
				bos.write(buffer, 0, bytesRead);
			}

			bos.flush(); // 确保所有数据都已写入响应
		}
	}






	/**
	 * 校验文件
	 * @param multipartFile     文件
	 * @param fileUploadBizEnum 业务类型
	 */
	private void validFile(MultipartFile multipartFile, FileUploadBizEnum fileUploadBizEnum) {
		// 文件大小
		long fileSize = multipartFile.getSize();
		// 文件后缀
		String fileSuffix = FileUtil.getSuffix(multipartFile.getOriginalFilename());
		final long LIMIT = 20 * 1024 * 1024L;
		if (fileSize > LIMIT) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件大小不能超过20MB");
		}
		if (!Arrays.asList("jpeg", "jpg", "svg", "png", "webp", "JPG").contains(fileSuffix)) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件类型错误");
		}
	}


	/**
	 * @param uploadFileRequest	业务名称
	 * @param multipartFile	文件
	 * @return	文件后缀路径
	 */
	private String getFilePath(MultipartFile multipartFile, UploadFileRequest uploadFileRequest) {
		// 获取业务名称
		String biz = uploadFileRequest.getBiz();
		FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
		if (fileUploadBizEnum == null) {
			throw new BusinessException(ErrorCode.PARAMS_ERROR, "业务名称错误");
		}
		// 校验文件
		validFile(multipartFile, fileUploadBizEnum);
		// 文件目录：根据业务、用户来划分
		String uuid = RandomStringUtils.randomAlphabetic(8);
		String filename = uuid + "-" + multipartFile.getOriginalFilename();
		return String.format("%s/%s", fileUploadBizEnum.getValue(), filename);
	}


}
